function [bestBox, E1, E2] = ml_findBox1g(kpAssign, imH, imW, binWeights, gtWin, szRange)
% function [bestBox, E1, E2] = ml_findBox1g(kpAssign, nKeyPnt, binWeights, gtWin, szRange)
% Find a rectangle with the histogram that minimize the weighted sum
%   sum_i {binWeights(i)*hist(i)} + 
%       area(intersect(win, gtWin))/area(union(win,gtWin))
% Inputs:
%   kpAssign: a matrix for histogram bin assignments of keypoints. 
%       kpAssign(i,1), kpAssign(i,2), kpAssign(i,3) are the row#, col#,
%       bin#. 1 <= row# <= imH; 1 <= col# <= imW; 1 <= bin# <= k, where k
%       is the length of binWeights.
%   imH, imW: height and width of the image
%   binWeights: a k*1 vector of weights for histogram bins, weights can be
%       negative
%   gtWin: a 4*1 vector for [left, top, right, bottom] of the gt window
%	szRange: a 4*1 vector for [wmin; wmax; hmin; hmax], range for width and height 
% Output:
%   bestBox: a 4*1 vector for the left, top, right, bottom of the rectangle
%   E1: sum_i {binWeights(i)*hist(i)}
%   E2: area(intersect(win, gtWin))/area(union(win,gtWin))
% Examples:
%   imAssign = ones(100, 200);
%   imAssign(50:70, 80:120) = 2;
%   [imH, imW] = size(imAssign);
%   kpAssign = [IY(:), IX(:), imAssign(:)];
%   [bestBox, e1, e2] = ml_findBox1g(kpAssign, imH, imW, [1;-0.01], [80;50; 120;
%   70]); ==> should get [80; 50; 120; 70];
%   [bestBox, e1, e2] = ml_findBox1g(kpAssign, imH, imW, [0.000001;-0.01], [80;50;
%   120; 70]); ==> should get [1 1 200 100];
% Note: This is for structure output learning code
% See also: ml_findBox1b  
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 22 Jan 09

if size(binWeights, 2) ~= 1
    error('binWeights should be a column vector');
end;

nHistBin = size(binWeights, 1);
if sum(kpAssign(:) <= 0) > 0
    error('entries of kpAssign should be positive integers');
end
if sum(kpAssign(:,1) > imH) > 0
    error('entries of kpAssign(:,1) must be <= imH');
end;
if sum(kpAssign(:,2) > imW) > 0
    error('entries of kpAssign(:,2) must be <= imW');
end;
if sum(kpAssign(:,3) > nHistBin) > 0
    error('entries of kpAssign(:,3) must be smaller than the length of binWeights');
end

if exist('szRange','var') && ~isempty(szRange)
    if (size(szRange, 1) ~= 4) || (size(szRange, 2) ~= 1)
        error('szRange should be a 4*1 vector');
    end;

    if (szRange(1) > szRange(2)) || (szRange(3) > szRange(4)) ||...
        szRange(2) <= 0 || szRange(4) <=0 || ...
        szRange(1) > imW || szRange(3) > imH
        error('invalid size range');
    end;
end;

if ~exist('szRange', 'var') || isempty(szRange)
    szRange = [0; imW; 0; imH];
end;

if ~exist('gtWin', 'var') || isempty(gtWin)
    gtWin = [-1; -1; -1; -1];
end;

[bestBox, E1, E2] = m_mexFindBox1g(kpAssign, imH, imW, binWeights, gtWin, szRange);    
    




