function [bestBox, E] = ml_findBox1i(kpAssign, imH, imW, binWeights, ...
    targetHist, C, szRange, tolFac)
% function [bestBox, E] = ml_findBox1i(kpAssign, imH, imW, binWeights, ...
%    targetHist)
% function [bestBox, E] = ml_findBox1i(kpAssign, imH, imW, binWeights, ...
%    targetHist, C)
% function [bestBox, E] = ml_findBox1i(kpAssign, imH, imW, binWeights, ...
%    targetHist, C, szRange)
% function [bestBox, E] = ml_findBox1i(kpAssign, imH, imW, binWeights, ...
%    targetHist, C, szRange, tolFac)
% Find a rectangle with the histogram that minimize the weighted sum
%   sum_i {binWeights(i)*|hist(i) - targetHist(i)|^2} - C*area(box)
% Inputs:
%   kpAssign: a matrix for histogram bin assignments of keypoints. 
%       kpAssign(i,1), kpAssign(i,2), kpAssign(i,3) are the row#, col#,
%       bin#. 1 <= row# <= imH; 1 <= col# <= imW; 1 <= bin# <= k, where k
%       is the length of binWeights.
%   imH, imW: height and width of the image
%   binWeights: a k*1 vector of weights for histogram bins, weights can be
%       negative
%   targetHist: a k*1 for target histogram, entries must be nonegative integers
%   C: param for trade-off for box with large area. C must be non-negative.
%	szRange: a 4*1 vector for [wmin; wmax; hmin; hmax], range for width and height
%   tolFac: for stopping early. tolFac is from 0 to 1.
%   tolFactor = 0: stop when global optimum is reach
%       tolFac > 0: stop when the difference between the global optimum
%       and the best known is < tolFac of the best known value.
% Output:
%   bestBox: a 4*1 vector for the left, top, right, bottom of the rectangle
%   E: energy
% Examples:
%   imH = 100; imW = 200;
%   imAssign = ones(imH, imW);
%   imAssign(50:70,100:150) = 2;
%   [IX,IY] = meshgrid(1:imW, 1:imH);
%   kpAssign = [IY(:), IX(:), imAssign(:)];
%   [bbox, E] = ml_findBox1i(kpAssign, imH, imW, [1; 1], [40;60]);
%   
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 9 May 2009

if size(binWeights, 2) ~= 1
    error('binWeights should be a column vector');
end;

if size(targetHist, 2) ~= 1
    error('targetHist should be a column vector');
end;

if size(binWeights, 1) ~= size(targetHist,1)
    error('weight vector and target histogram should be the same size');
end;



nHistBin = size(binWeights, 1);
if sum(kpAssign(:) <= 0) > 0
    error('entries of kpAssign should be positive integers');
end

if sum(kpAssign(:,1) > imH) > 0
    error('entries of kpAssign(:,1) must be <= imH');
end;

if sum(kpAssign(:,2) > imW) > 0
    error('entries of kpAssign(:,2) must be <= imW');
end;
if sum(kpAssign(:,3) > nHistBin) > 0
    error('entries of kpAssign(:,3) must be smaller than the length of binWeights');
end

if ~exist('C', 'var') || isempty(C);
    C = 0;
elseif C < 0
    error('C must be a non-negative number');
end;

if ~exist('szRange', 'var') || isempty(szRange)
    szRange = [0; imW; 0; imH];
elseif (size(szRange, 1) ~= 4) || (size(szRange, 2) ~= 1)
    error('szRange should be a 4*1 vector');
elseif (szRange(1) > szRange(2)) || (szRange(3) > szRange(4)) ||...
        szRange(2) <= 0 || szRange(4) <=0 || ...
        szRange(1) > imW || szRange(3) > imH
    error('invalid size range');
end;

if ~exist('tolFac', 'var') || isempty(tolFac)
    tolFac = 0;
elseif tolFac < 0 || tolFac > 1
    error('tolFac must be between 0 and 1');
end;


[bestBox, E] = m_mexFindBox1i(kpAssign, imH, imW, binWeights, ...
    targetHist, C, szRange, tolFac);
    




