function [bestBox, E] = ml_findBox1j(kpAssign, imH, imW, B, szRange, alpha, tolFac, option)
% [bestBox, E] = ml_findBox1j(kpAssign, imH, imW, B, szRange, tolFac, option)
% Find a rectangle with the histogram that minimize one of the followings:
%    i) 1/(1+alpha)*||h||^2 - ||B'*h||^2;
%   ii) - ||B'*h||^2/(||h||^2 + alpha).
% Inputs:
%   kpAssign: a matrix for histogram bin assignments of keypoints. 
%       kpAssign(i,1), kpAssign(i,2), kpAssign(i,3) are the row#, col#,
%       bin#. 1 <= row# <= imH; 1 <= col# <= imW; 1 <= bin# <= k, where k
%       is the length of binWeights.
%   imH, imW: height and width of the image
%   B: the nBin*nPC matrix for PCA basis.   
%	szRange: a 4*1 vector for [wmin; wmax; hmin; hmax], range for width and height
%   alpha: must be a nonnegative number. The bigger alpha, the larger
%       the returned rectangle tends to be.
%   tolFac: tolerance factor
%   option: option for which of the two things to optimize. 
%       If option is 1 this optimizes i) described above. 
%       This optimizes ii) otherwise.
% Output:
%   bestBox: a 4*1 vector for the left, top, right, bottom of the rectangle
%   E: energy
% Examples:
%   imH = 100; imW = 200;
%   imAssign = ones(imH, imW);
%   imAssign(40:70,100:150) = 2;
%   imAssign(50:60,110:140) = 3;
%   B = [1/sqrt(3); 1/sqrt(3); 1/sqrt(3)];
%   [IX,IY] = meshgrid(1:imW, 1:imH);
%   kpAssign = [IY(:), IX(:), imAssign(:)];
%   tic; [bbox,E]=ml_findBox1j(kpAssign, imH, imW, B, [], 0.0, 0, 2), toc;
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 12 Oct 09


[nHistBin, nPC] = size(B);
if (nPC > nHistBin)
    error('B should not have more columns than rows');
end;

if sum(kpAssign(:) <= 0) > 0
    error('entries of kpAssign should be positive integers');
end

if sum(kpAssign(:,1) > imH) > 0
    error('entries of kpAssign(:,1) must be <= imH');
end;

if sum(kpAssign(:,2) > imW) > 0
    error('entries of kpAssign(:,2) must be <= imW');
end;
if sum(kpAssign(:,3) > nHistBin) > 0
    error('entries of kpAssign(:,3) must be smaller than the length of binWeights');
end

if isempty(szRange)
    szRange = [1; imW; 1; imH];
end
if (size(szRange, 1) ~= 4) || (size(szRange, 2) ~= 1)
    error('szRange should be a 4*1 vector');
end;

if (szRange(1) > szRange(2)) || (szRange(3) > szRange(4)) ||...
    szRange(2) <= 0 || szRange(4) <=0 || ...
    szRange(1) > imW || szRange(3) > imH
    error('invalid size range');
end;

if ~exist('alpha', 'var') || isempty(alpha)
    alpha = 0;
end;

if alpha < 0
    error('alpha must be non-negative');
end;


if ~exist('tolFac', 'var') || isempty(tolFac)
    tolFac = 0;
end;
if ~exist('option', 'var') || isempty(option)
    option = 2;
end;
[bestBox, E] = m_mexFindBox1j(kpAssign, imH, imW, B, szRange, alpha, tolFac, option);
    




