function [win1, win2, E] = ml_findBoxes2c(kpAssign1, imH1, imW1, szRange1, ...
    kpAssign2, imH2, imW2, szRange2, nHistBin, alpha, tolFac)
% [win1, win2, E] = ml_findBoxes2c(kpAssign1, imH1, imW1, szRange1, ...
%    kpAssign2, imH2, imW2, szRange2, nHistBin, alpha, tolFac)
% Find two rectangles in two images that minimize the following:
%   -h1^Th2/(|h1|_2|h2|_2 + alpha)
% Inputs:
%   kpAssign: a matrix for histogram bin assignments of keypoints. 
%       kpAssign(i,1), kpAssign(i,2), kpAssign(i,3) are the row#, col#,
%       bin#. 1 <= row# <= imH; 1 <= col# <= imW; 1 <= bin# <= k, where k
%       is nHistBin.
%   imH, imW: height and width of the image
%	szRange: a 4*1 vector for [wmin; wmax; hmin; hmax], range for width and height
%   alpha: must be a nonnegative number. The bigger alpha, the larger
%       the returned rectangle tends to be.
%   tolFac: tolerance factor
% Output:
%   win1, win2: 4*1 vectors for the left, top, right, bottom of the rectangles
%   E: energy
% Examples:
%   imH1 = 10; imW1 = 20;
%   imAssign1 = ones(imH1, imW1);
%   imAssign1(4:7,10:15) = 2;
%   imAssign1(5:6,11:14) = 3;
%   [IX,IY] = meshgrid(1:imW1, 1:imH1);
%   kpAssign1 = [IY(:), IX(:), imAssign1(:)];
%   imH2 = 20; imW2 = 10;
%   imAssign2 = 4*ones(imH2, imW2);
%   imAssign2(10:15, 4:7) = 2;
%   imAssign2(11:14, 5:6) = 3;
%   [IX,IY] = meshgrid(1:imW2, 1:imH2);
%   kpAssign2 = [IY(:), IX(:), imAssign2(:)];
%   tic; [win1, win2, E]=ml_findBoxes2c(kpAssign1, imH1, imW1, [], kpAssign2, imH2, imW2, [], 4, 2.0, 0), toc;
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 12 Oct 09

szRange1 = checkData(kpAssign1, imH1, imW1, szRange1, nHistBin);
szRange2 = checkData(kpAssign2, imH2, imW2, szRange2, nHistBin);

    
if ~exist('alpha', 'var') || isempty(alpha)
    alpha = 0;
end;

if alpha < 0
    error('alpha must be non-negative');
end;

if ~exist('tolFac', 'var') || isempty(tolFac)
    tolFac = 0;
end;

if (nargout == 3)
    [win1, win2, E] = m_mexFindBoxes2c(kpAssign1, imH1, imW1, szRange1, ...
        kpAssign2, imH2, imW2, szRange2, nHistBin, alpha, tolFac);
else
    [win1, win2] = m_mexFindBoxes2c(kpAssign1, imH1, imW1, szRange1, ...
        kpAssign2, imH2, imW2, szRange2, nHistBin, alpha, tolFac);
end;


function szRange = checkData(kpAssign, imH, imW, szRange, nHistBin)
    if sum(kpAssign(:) <= 0) > 0
        error('entries of kpAssign should be positive integers');
    end

    if sum(kpAssign(:,1) > imH) > 0
        error('entries of kpAssign(:,1) must be <= imH');
    end;

    if sum(kpAssign(:,2) > imW) > 0
        error('entries of kpAssign(:,2) must be <= imW');
    end;
    if sum(kpAssign(:,3) > nHistBin) > 0
        error('entries of kpAssign(:,3) must be smaller than the length of binWeights');
    end

    if isempty(szRange)
        szRange = [1; imW; 1; imH];
    end
    if (size(szRange, 1) ~= 4) || (size(szRange, 2) ~= 1)
        error('szRange should be a 4*1 vector');
    end;

    if (szRange(1) > szRange(2)) || (szRange(3) > szRange(4)) ||...
        szRange(2) <= 0 || szRange(4) <=0 || ...
        szRange(1) > imW || szRange(3) > imH
        error('invalid size range');
    end;
    