#include <iostream>
#include <fstream>
#include <cmath>
#include <queue>
#include <string>
#include <sstream>
#include "m_malloc.h"
#include<stdlib.h>
#include "m_utils.h"
#include "time.h"

using namespace std;

/* Compute Integral image
 * im: 2D array of int. Each entry must be a POSITIVE integer
 * which is less than or equal to nHistBin
 * Compute Integral Images for histograms using dynamic programming.
 */
int*** cmpIntegralIms(int **im, int imH, int imW, int nHistBin){
   int*** integralIms = buildArrayThree<int>(imH, imW, nHistBin);

   integralIms[0][0][im[0][0]-1] = 1;
   for (int j=1; j < imW; j++){
      for (int u=0; u < nHistBin; u++){
         integralIms[0][j][u] = integralIms[0][j-1][u];
      }
      integralIms[0][j][im[0][j] - 1]++;
   }

   for (int i=1; i < imH; i++){
      for (int u=0; u < nHistBin; u++){
         integralIms[i][0][u] = integralIms[i-1][0][u];
      }
      integralIms[i][0][im[i][0] - 1]++;
   }


   for (int i=1; i < imH; i++){
      for (int j=1; j < imW; j++){
         for (int k=0; k < nHistBin; k++){
            integralIms[i][j][k] = integralIms[i][j-1][k] + integralIms[i-1][j][k]
            - integralIms[i-1][j-1][k];
         }
         integralIms[i][j][im[i][j] - 1]++;
      }
   }
   return integralIms;
}

/**
 * Compute the integral images, similar to cmpIntegralIms,
 * but the input format is different
 */
int***
cmpIntegralIms1(int **kpAssign, int nKeyPnt, int imH, int imW, int nHistBin){
   vector<vector<vector<int> > > grdKpnts(imH);

   for (int i=0; i < imH; i++){
	   grdKpnts[i].resize(imW);
   }

   int col, row, bin;
   for (int i=0; i < nKeyPnt; i++){
      row = kpAssign[i][0] - 1;
      col = kpAssign[i][1] - 1;
      bin = kpAssign[i][2] - 1;

      grdKpnts[row][col].push_back(bin);
   }

   int*** integralIms = buildArrayThree<int>(imH, imW, nHistBin);

   for (unsigned int k=0; k < grdKpnts[0][0].size(); k++){
      integralIms[0][0][grdKpnts[0][0][k]]++;
   }

   for (int j=1; j < imW; j++){
      for (int u=0; u < nHistBin; u++){
         integralIms[0][j][u] = integralIms[0][j-1][u];
      }
      for (unsigned int k=0; k < grdKpnts[0][j].size(); k++){
         integralIms[0][j][grdKpnts[0][j][k]]++;
      }
   }

   for (int i=1; i < imH; i++){
      for (int u=0; u < nHistBin; u++){
         integralIms[i][0][u] = integralIms[i-1][0][u];
      }
      for (unsigned int k=0; k < grdKpnts[i][0].size(); k++){
         integralIms[i][0][grdKpnts[i][0][k]]++;
      }
   }


   for (int i=1; i < imH; i++){
      for (int j=1; j < imW; j++){
         for (int k=0; k < nHistBin; k++){
            integralIms[i][j][k] = integralIms[i][j-1][k] + integralIms[i-1][j][k]
            - integralIms[i-1][j-1][k];
         }
         for (unsigned int k=0; k < grdKpnts[i][j].size(); k++){
            integralIms[i][j][grdKpnts[i][j][k]]++;
         }
      }
   }
   return integralIms;
}



/* Compute Only two integral images, one for bins with positive weights
 * one for bins with negative weights.
 * im: 2D array of int. Each entry must be a POSITIVE integer
 * which is less than or equal to nHistBin
 * Compute Integral Images for histograms using dynamic programming.
 */
double*** cmpIntegralIms2(int **im, int imH, int imW, double *weights, int nHistBin){
   // only 2 integral images, the first one is for positive weights
   // the second one is for negative weights
   double*** integralIms = buildArrayThree<double>(imH, imW, 2);


   if (weights[im[0][0] - 1] > 0){
      integralIms[0][0][0] = weights[im[0][0] - 1];
   } else {
      integralIms[0][0][1] = -weights[im[0][0] - 1];
   }

   for (int j=1; j < imW; j++){
      for (int u=0; u < 2; u++){
         integralIms[0][j][u] = integralIms[0][j-1][u];
      }
      if (weights[im[0][j] - 1] > 0){
         integralIms[0][j][0] += weights[im[0][j] - 1];
      } else {
         integralIms[0][j][1] -= weights[im[0][j] - 1];
      }
   }

   for (int i=1; i < imH; i++){
      for (int u=0; u < 2; u++){
         integralIms[i][0][u] = integralIms[i-1][0][u];
      }
      if (weights[im[i][0] - 1] > 0){
         integralIms[i][0][0] += weights[im[i][0] - 1];
      } else {
         integralIms[i][0][1] -= weights[im[i][0] - 1];
      }
   }

   for (int i=1; i < imH; i++){
      for (int j=1; j < imW; j++){
         for (int k=0; k < 2; k++){
            integralIms[i][j][k] = integralIms[i][j-1][k] + integralIms[i-1][j][k]
            - integralIms[i-1][j-1][k];
         }
         if (weights[im[i][j] - 1] > 0){
            integralIms[i][j][0] += weights[im[i][j] - 1];
         } else {
            integralIms[i][j][1] -= weights[im[i][j] - 1];
         }
      }
   }
   return integralIms;
}

/**
 * Compute the integral images, similar to cmpIntegralIms2,
 * but the input format is different.
 */
double*** cmpIntegralIms3(int **kpAssign, int nKeyPnt, int imH, int imW,
                          double *weights, int nHistBin){
   // only 2 integral images, the first one is for positive weights
   // the second one is for negative weights
   double** posIm = buildMatrix<double>(imH, imW);
   double** negIm = buildMatrix<double>(imH, imW);

   int col, row, bin;
   for (int i=0; i < nKeyPnt; i++){
      row = kpAssign[i][0] - 1;
      col = kpAssign[i][1] - 1;
      bin = kpAssign[i][2] - 1;

      if (weights[bin] > 0){
         posIm[row][col] += weights[bin];
      } else {
         negIm[row][col] -= weights[bin];
      }
   }

   //Refine the posIm and negIm for tighter bounds
   for (int i=0; i < imH; i++){
      for (int j=0; j < imW; j++){
         if (posIm[i][j] > negIm[i][j]){
            posIm[i][j] -= negIm[i][j];
            negIm[i][j] = 0;
         } else {
            negIm[i][j] -= posIm[i][j];
            posIm[i][j] = 0;
         }
      }
   }

   double*** integralIms = cmpIntegralIms3_helper(imH, imW, posIm, negIm);

   destroyMatrix<double>(posIm, imH, imW);
   destroyMatrix<double>(negIm, imH, imW);
   return integralIms;
}

double ***cmpIntegralIms3_helper(int & imH, int & imW, double **& posIm, double **& negIm)
{
    double ***integralIms = buildArrayThree<double>(imH, imW, 2);
    integralIms[0][0][0] = posIm[0][0];
    integralIms[0][0][1] = negIm[0][0];
    for(int j = 1;j < imW;j++){
        integralIms[0][j][0] = integralIms[0][j - 1][0] + posIm[0][j];
        integralIms[0][j][1] = integralIms[0][j - 1][1] + negIm[0][j];
    }
    for(int i = 1;i < imH;i++){
        integralIms[i][0][0] = integralIms[i - 1][0][0] + posIm[i][0];
        integralIms[i][0][1] = integralIms[i - 1][0][1] + negIm[i][0];
    }
    for(int i = 1;i < imH;i++){
        for(int j = 1;j < imW;j++){
            integralIms[i][j][0] = integralIms[i][j - 1][0] +
               integralIms[i - 1][j][0] - integralIms[i - 1][j - 1][0] + posIm[i][j];
            integralIms[i][j][1] = integralIms[i][j - 1][1] +
               integralIms[i - 1][j][1] - integralIms[i - 1][j - 1][1] + negIm[i][j];
        }
    }
    return integralIms;
}




/* Compute Integral images
 * pixVals: 2D array of double.
 * Compute 2 Integral Images one for positive and one for negative.
 * Similar to cmpIntegralIms2 and cmpIntegralIms3 except for the input format
 */
double*** cmpIntegralIms4(double **pixVals, int imH, int imW){
   double** posIm = buildMatrix<double>(imH, imW);
   double** negIm = buildMatrix<double>(imH, imW);

   //Refine the posIm and negIm for tighter bounds
   for (int i=0; i < imH; i++){
      for (int j=0; j < imW; j++){
         if (pixVals[i][j] >= 0){
            posIm[i][j] = pixVals[i][j];
         } else {
            negIm[i][j] = - pixVals[i][j];
         }
      }
   }

   double*** integralIms = cmpIntegralIms3_helper(imH, imW, posIm, negIm);

   destroyMatrix<double>(posIm, imH, imW);
   destroyMatrix<double>(negIm, imH, imW);
   return integralIms;
}

double **cmpIntegralIms_1D(double *pixVals, int len){
   double **integralIms = buildMatrix<double>(2, len);
   if (pixVals[0] > 0) {
      integralIms[0][0] = pixVals[0];
      integralIms[1][0] = 0;
   } else {
      integralIms[0][0] = 0;
      integralIms[1][0] = -pixVals[0];
   }

   for (int i=1; i<len; i++){
      if (pixVals[i] >=0) {
         integralIms[0][i] = integralIms[0][i-1] + pixVals[i];
         integralIms[1][i] = integralIms[1][i-1];
      } else {
         integralIms[0][i] = integralIms[0][i-1];
         integralIms[1][i] = integralIms[1][i-1] - pixVals[i];
      }
   }
   return integralIms;
}

void cmpIntegralIms_1D_1(const double *pixVals, const int len, double *integralIm){
   integralIm[0] = pixVals[0];
   for (int i=1; i < len; i++) integralIm[i] = integralIm[i-1] + pixVals[i];
}


int** imAssign2kpAssign(int **imAssign1, int imH1, int imW1){
   int **kpAssign = buildMatrix<int>(imH1*imW1, 3);
   int cnt = 0;
   for (int i = 0; i< imH1; i++){
      for (int j= 0; j < imW1; j++){
         kpAssign[cnt][0] = i + 1;
         kpAssign[cnt][1] = j + 1;
         kpAssign[cnt][2] = imAssign1[i][j];
         cnt++;
      }
   }
   return kpAssign;
}

double **kpAssign2pixVals(int **kpAssign, int nKeyPnt, int imH, int imW, double *weights){

   double** pixVals = buildMatrix<double>(imH, imW);
   int col, row, bin;
   for (int i=0; i < nKeyPnt; i++){
      row = kpAssign[i][0] - 1;
      col = kpAssign[i][1] - 1;
      bin = kpAssign[i][2] - 1;
      pixVals[row][col] += weights[bin];
   }
   return pixVals;
}

/* The lower bound is obtained by minimize:
 * abs(h1 - h_target) - C*h1 with h1_min <= h1 <= h1_max
 * This assume 0 <= C <= 1
 */
double getLowerBnd1_helper(int h1_min, int h1_max, int h_target, double C){
   double lb;
   int h1;
   if (h_target < h1_min){
      h1 = h1_min;
   }else if (h_target > h1_max){
      h1 = h1_max;
   } else {
      h1 = h_target;
   }
   lb = abs(h1 - h_target) - C*(h1 + h_target);
   return lb;
}

/* The lower bound is obtained by minimize:
 * abs(h1 - h2) - C*(h1 + h2)
 * s.t. h1_min <= h1 <= h1_max and h2_min <= h2 <= h2_max
 * This assume 0 <= C <= 1 and h1_max <= h2_max;
 */
double getLowerBnd2_helper2(int h1_min, int h1_max, int h2_min, int h2_max, double C){
   int h1, h2;

   h1 = h1_max;
   if (h2_min <= h1_max){
      h2 = h1_max;
   } else {
      h2 = h2_min;
   }
   double lb = abs(h1 - h2) - C*(h1 + h2);
   return lb;
}

/* The lower bound is obtained by minimize:
 * abs(h1 - h2) - C*(h1 + h2)
 * s.t. h1_min <= h1 <= h1_max and h2_min <= h2 <= h2_max
 * This assume 0 <= C <= 1
 */
double getLowerBnd2_helper(int h1_min, int h1_max, int h2_min, int h2_max, double C){
   double lb;
   if (h1_max <= h2_max){
      lb = getLowerBnd2_helper2(h1_min, h1_max, h2_min, h2_max, C);
   } else {
      lb = getLowerBnd2_helper2(h2_min, h2_max, h1_min, h1_max, C);
   }
   return lb;
}

double getLowerBnd1(int* hist1_min, int* hist1_max, int* hist_target, double *weights,
                    int nHistBin, double C){
   double lb = 0;

   for (int k=0; k < nHistBin; k++){
      lb = lb + weights[k]*getLowerBnd1_helper(hist1_min[k], hist1_max[k], hist_target[k], C);
   }
   return lb;
}

double getLowerBnd2(int* hist1_min, int* hist1_max, int* hist2_min, int* hist2_max, double *weights,
                    int nHistBin, double C){
   double lb = 0;

   for (int k=0; k < nHistBin; k++){
      lb = lb + weights[k]*getLowerBnd2_helper(hist1_min[k], hist1_max[k], hist2_min[k], hist2_max[k], C);
   }
   return lb;
}

double getLowerBnd3(int *hist_min, int *hist_max, double *binWeights, int nHistBin){
   double lb = 0;
   for (int i=0; i < nHistBin; i++){
      if (binWeights[i] < 0){
         lb += binWeights[i]*hist_max[i];
      } else if (binWeights[i] > 0){
         lb += binWeights[i]*hist_min[i];
      }
   }
   return lb;
}


double cmpEnergy(int* hist1, int* hist2, double *weights, int nHistBin, double C){
   double energy = 0;
   for (int k= 0; k < nHistBin; k++){
      energy += weights[k]*(abs(hist1[k] - hist2[k]) - C*(hist1[k] + hist2[k]));
   }
   return energy;
}

double cmpEnergy2(int *hist1, double *weights, int nHistBin){
   double energy = 0;
   for (int i=0; i < nHistBin; i++){
      energy += hist1[i]*weights[i];
   }
   return energy;
}

double cmpEnergy3(int *hist1, double **B, int nHistBin, int nPC, double alpha){
   double energy = 0;
   for (int i=0; i < nHistBin; i++){
      energy += hist1[i]*hist1[i];
   }

   energy = energy/(1+alpha);

   for (int j= 0; j < nPC; j++){
      double projVal = 0;
      for (int i= 0; i < nHistBin; i++){
         projVal += B[j][i]*hist1[i];
      }
      energy -= projVal*projVal;
   }
   return energy;
}

double cmpEnergy4(int *hist1, double **B, int nHistBin, int nPC, double alpha){
   double numer = 0, denom = 0;
   for (int i=0; i < nHistBin; i++){
      denom += hist1[i]*hist1[i];
   }
   for (int j= 0; j < nPC; j++){
      double projVal = 0;
      for (int i= 0; i < nHistBin; i++){
         projVal += B[j][i]*hist1[i];
      }
      numer -= projVal*projVal;
   }
   return numer/(denom + alpha);
}

double cmpEnergy5(int *hist1, int *hist2, int nHistBin, double alpha){
   double h1_norm = 0.0, h2_norm = 0.0, h1h2 = 0.0;
   for (int i=0; i <nHistBin; i++){
      h1_norm += hist1[i]*hist1[i];
      h2_norm += hist2[i]*hist2[i];
      h1h2 += hist1[i]*hist2[i];
   }
   double energy = -h1h2/(sqrt(h1_norm*h2_norm) + alpha);
   return energy;
}


Window::Window(int ul_x_, int ul_y_, int lr_x_, int lr_y_):
   ul_x(ul_x_), ul_y(ul_y_), lr_x(lr_x_), lr_y(lr_y_){
};

Window::Window(){
   ul_x = -1;
   ul_y = -1;
   lr_x = -1;
   lr_y = -1;
};

Window::Window(const Window &win){
   ul_x = win.ul_x;
   ul_y = win.ul_y;
   lr_x = win.lr_x;
   lr_y = win.lr_y;
}

Window::Window(const Window &win, Res winRes){
   ul_x = win.ul_x*winRes.x;
   ul_y = win.ul_y*winRes.y;
   lr_x = win.lr_x*winRes.x;
   lr_y = win.lr_y*winRes.y;
};


Window
Window::intersectWin(Window otherWin){
   // intersection with empty window
   if ((ul_x == -1) || (otherWin.ul_x == -1)){
      return Window();
   }
   Window inter1(max(ul_x, otherWin.ul_x),
                 max(ul_y, otherWin.ul_y),
                 min(lr_x, otherWin.lr_x),
                 min(lr_y, otherWin.lr_y));
   if ((inter1.ul_x > inter1.lr_x) || (inter1.ul_y > inter1.lr_y))
      return Window();
   else return inter1;

}

Window
Window::unionWin(Window otherWin){
   if (ul_x == -1){
      return otherWin;
   } else if (otherWin.ul_x == -1){
      return Window(ul_x, ul_y, lr_x, lr_y);
   } else {
      return Window(min(ul_x, otherWin.ul_x),
                 min(ul_y, otherWin.ul_y),
                 max(lr_x, otherWin.lr_x),
                 max(lr_y, otherWin.lr_y));
   }
}



int* Window::getHist(int*** integralIms, int nHistBin){
   int* hist = buildVector<int>(nHistBin);
   if (ul_x > -1){
      if ((ul_x > 0) && (ul_y > 0)){
         for (int k=0; k <nHistBin; k++){
            hist[k] = integralIms[lr_y][lr_x][k] + integralIms[ul_y-1][ul_x-1][k] -
               integralIms[lr_y][ul_x-1][k] - integralIms[ul_y-1][lr_x][k];
         }
      } else if (ul_x > 0) {
         for (int k=0; k <nHistBin; k++){
            hist[k] = integralIms[lr_y][lr_x][k] - integralIms[lr_y][ul_x-1][k];
         }
      } else if (ul_y > 0) {
         for (int k=0; k <nHistBin; k++){
            hist[k] = integralIms[lr_y][lr_x][k] - integralIms[ul_y-1][lr_x][k];
         }
      } else {
         for (int k=0; k <nHistBin; k++){
            hist[k] = integralIms[lr_y][lr_x][k];
         }
      }
   }
   return hist;
}

double* Window::getPosNegScore(double ***integralIms){
   double *hist = buildVector<double>(2);
   int nHistBin = 2;

   if (ul_x > -1){
      if ((ul_x > 0) && (ul_y > 0)){
         for (int k=0; k < nHistBin; k++){
            hist[k] = integralIms[lr_y][lr_x][k] + integralIms[ul_y-1][ul_x-1][k] -
               integralIms[lr_y][ul_x-1][k] - integralIms[ul_y-1][lr_x][k];
         }
      } else if (ul_x > 0) {
         for (int k=0; k <nHistBin; k++){
            hist[k] = integralIms[lr_y][lr_x][k] - integralIms[lr_y][ul_x-1][k];
         }
      } else if (ul_y > 0) {
         for (int k=0; k <nHistBin; k++){
            hist[k] = integralIms[lr_y][lr_x][k] - integralIms[ul_y-1][lr_x][k];
         }
      } else {
         for (int k=0; k <nHistBin; k++){
            hist[k] = integralIms[lr_y][lr_x][k];
         }
      }
   }
   return hist;
}


int* Window::getHist(int ***integralIms, int nHistBin, Res winRes){
   Window newWin(ul_x*winRes.x, ul_y*winRes.y, lr_x*winRes.x, lr_y*winRes.y);
   return newWin.getHist(integralIms, nHistBin);
}


int Window::getMaxDim(){
   return max(lr_x - ul_x, lr_y - ul_y);
};

int Window::getArea(){
   if (ul_x == -1) return 0;
   return (lr_x - ul_x + 1)*(lr_y - ul_y + 1);
}

void Window::copy(Window *newWin){
   newWin->lr_x = lr_x;
   newWin->lr_y = lr_y;
   newWin->ul_x = ul_x;
   newWin->ul_y = ul_y;
}

void Window::split(Window *left, Window *right){
   if (lr_x - ul_x >= lr_y -  ul_y){ // split along horizontal dim
      int x1 = (ul_x + lr_x)/2;
      int x2 = x1 + 1;
      copy(left);
      left->lr_x = x1;
      copy(right);
      right->ul_x = x2;
   } else {
      int y1 = (ul_y + lr_y)/2;
      int y2 = y1+1;
      copy(left);
      left->lr_y = y1;
      copy(right);
      right->ul_y = y2;
   }
};

bool Window::operator ==(const Window& win){
   return ((ul_x == win.ul_x) && (ul_y == win.ul_y) &&
      (lr_x == win.lr_x) && (lr_y == win.lr_y));
};

string Window::str(){
   ostringstream rslt;
   rslt << "(" << ul_x << ", " << ul_y << ", " << lr_x << ", " << lr_y << ")";
   return rslt.str();
};




WindowPair::WindowPair(Window win1_, Window win2_){
   win1 = win1_;
   win2 = win2_;
}
WindowPair::WindowPair(){};


Window
WindowPair::getSmallestWin(){
   if ((win1.lr_x <= win2.ul_x) && (win1.lr_y <= win2.ul_y)){
      return Window(win1.lr_x, win1.lr_y, win2.ul_x, win2.ul_y);
   } else {
      return Window();
   }
}

Window
WindowPair::getLargestWin(){
   if ((win1.ul_x <= win2.lr_x) && (win1.ul_y <= win2.lr_y)){
      return Window(win1.ul_x, win1.ul_y, win2.lr_x, win2.lr_y);
   } else {
      return Window();
   }
}




Window
WindowPair::getRandomWin(){
   int rUL_x = getRandNum(win1.ul_x, win1.lr_x);
   int rUL_y = getRandNum(win1.ul_y, win1.lr_y);
   int rLR_x = getRandNum(win2.ul_x, win2.lr_x);
   int rLR_y = getRandNum(win2.ul_y, win2.lr_y);
   if ((rUL_x <= rLR_x) && (rUL_y <= rLR_y)){
      return Window(rUL_x, rUL_y, rLR_x, rLR_y);
   } else {
      return getLargestWin();
   }
}

int
WindowPair::getRandNum(int minNum, int maxNum){
   srand((unsigned int)time(0));
   return (rand()% (maxNum + 1- minNum)) + minNum;
}
WindowPair
WindowPair::getSubWinPair(int x, int y, int xDev, int yDev){
   WindowPair rslt;
   Window w1 = Window(win1.ul_x, win1.ul_y, win2.ul_x, win2.ul_y).getSubWin2(x, y, xDev, yDev);
   Window w2 = Window(win1.lr_x, win1.lr_y, win2.lr_x, win2.lr_y).getSubWin2(x, y, xDev, yDev);

   if (w1.ul_x == -1) // if w1 is empty
      w1 = Window(win1.ul_x, win1.ul_y, win1.ul_x, win1.ul_y);

   if (w2.ul_x == -1) // if w2 is empty
      w2 = Window(win2.lr_x, win2.lr_y, win2.lr_x, win2.lr_y);

   rslt.win1 = Window(w1.ul_x, w1.ul_y, w2.ul_x, w2.ul_y);
   rslt.win2 = Window(w1.lr_x, w1.lr_y, w2.lr_x, w2.lr_y);
   return rslt;
}

Window
Window::getSubWin2(int x, int y, int xDev, int yDev){
   Window rslt;
   rslt.ul_x = ul_x + (int)ceil(((double)lr_x - ul_x + 1)*(x-1)/xDev);
   rslt.lr_x = ul_x + (int)ceil(((double)lr_x - ul_x + 1)*x/xDev)  - 1;

   rslt.ul_y = ul_y + (int)ceil(((double)lr_y - ul_y + 1)*(y-1)/yDev);
   rslt.lr_y = ul_y + (int)ceil(((double)lr_y - ul_y + 1)*y/yDev) - 1;
   return rslt;
}


Window
Window::getSubWin(int x, int y, int xDev, int yDev){
   Window rslt;
   rslt.ul_x = ul_x + (int)ceil(((double)lr_x - ul_x + 1)*(x-1)/xDev);
   rslt.lr_x = ul_x + (int)ceil(((double)lr_x - ul_x + 1)*x/xDev)  - 1;

   rslt.ul_y = ul_y + (int)ceil(((double)lr_y - ul_y + 1)*(y-1)/yDev);
   rslt.lr_y = ul_y + (int)ceil(((double)lr_y - ul_y + 1)*y/yDev) - 1;
   if ((rslt.ul_x > rslt.lr_x) || (rslt.ul_y > rslt.lr_y)){
      return Window();
   } else {
      return rslt;
   }
}


int
WindowPair::getMaxDim(){
   return max(win1.getMaxDim(), win2.getMaxDim());
};

void WindowPair::copy(WindowPair *newWinPair){
   newWinPair->win1 = win1;
   newWinPair->win2 = win2;
}

void WindowPair::split(WindowPair *left, WindowPair *right){
   if (win1.getMaxDim() > win2.getMaxDim()) { //split the first rectangle
      left->win2 = win2;
      right->win2 = win2;
      win1.split(&left->win1, &right->win1);
   } else {
      left->win1 = win1;
      right->win1 = win1;
      win2.split(&left->win2, &right->win2);
   }
}

string WindowPair::str(){
   return win1.str() + " " + win2.str();
}


string
SizeRange::str(){
   ostringstream rslt;
   rslt << "(" << hmin << ", " << hmax << ", " << wmin << ", " << wmax << ")";
   return rslt.str();
}

bool findEnds(Res end1Range, Res end2Range, Res szRng, Res *ends){
   if (((end2Range.x - end1Range.y + 1) > szRng.y) ||
      ((end2Range.y - end1Range.x + 1) < szRng.x)){
       return false;
   } else {
      if ((end2Range.x - end1Range.x + 1) > szRng.y){
         ends->y = end2Range.x;
         ends->x = end2Range.x - szRng.y + 1;
      } else if ((end2Range.x - end1Range.x + 1) < szRng.x){
         ends->x = end1Range.x;
         ends->y = end1Range.x + szRng.x - 1;
      } else {
         ends->x = end1Range.x;
         ends->y = end2Range.x;
      }
      return true;
   }
}




bool isImAssignGood(int **imAssign, int imH, int imW){
   bool isGood = true;
   for (int i=0; i < imH; i++){
      for (int j=0; j < imW; j++){
         if (imAssign[i][j] < 1) isGood = false;
      }
   }
   return isGood;
}

bool isWithinTolFactor(double bestUb, double lb, double tolFactor){
   if ((bestUb - lb) <= tolFactor*abs(bestUb)){
      return true;
   } else {
      return false;
   }
}

int **readImMatrix(char *fileName, int *imH_, int *imW_, int *nHistBin_){
   fstream myfile;
   myfile.open(fileName, fstream::in);
   if (myfile.fail()) cout << "could not open " << fileName << endl;
   int imH, imW, nHistBin;
   double tmp;
   myfile >> imH >> imW >> nHistBin;
   cout << imH << " " << imW << endl;
   int **imAssign = buildMatrix<int>(imH, imW);

   for (int i =0; i < imH; i++){
      for (int j = 0; j < imW; j++){
         myfile >> tmp;
         imAssign[i][j] = (int)floor(tmp);
         printf("%2d ", imAssign[i][j]);
      }
      cout << endl;
   };
   myfile.close();
   *imH_ = imH;
   *imW_ = imW;
   *nHistBin_ = nHistBin;
   return imAssign;
}


void
NumPair::split(NumPair *left, NumPair *right){
   int mid = (u+l)/2;
   left->l  = l;
   left->u  = mid;
   right->l = mid+1;
   right->u = u;
}

NumPair
NumPair::intersectNP(NumPair np){
   if ((l == -1) || (np.l == -1)) return NumPair();
   NumPair interPair(max(l, np.l), min(u, np.u));
   if (interPair.u < interPair.l) return NumPair();
   else return interPair;
}

NumPair
NumPair::unionNP(NumPair np){
   if (l == -1) return np;
   if (np.l == -1) return NumPair(l, u);
   return NumPair(min(l, np.l), max(u, np.u));
}

int
NumPair::getLen(){
   if (l==-1) return 0;
   else return (u-l+1);
}


double*
NumPair::getPosNegScore(double **integralIms){
   double *pos_neg = buildVector<double>(2);

   if (l > 0){
      pos_neg[0] = integralIms[0][u] - integralIms[0][l-1];
      pos_neg[1] = integralIms[1][u] - integralIms[1][l-1];
   } else if (l == 0) {
      pos_neg[0] = integralIms[0][u];
      pos_neg[1] = integralIms[1][u];
   }

   return pos_neg;
}

double
NumPair::F1Loss(NumPair np){
   if (isEmpty() && np.isEmpty()) return 0;
   else if (isEmpty() || np.isEmpty()) return 1;
   else{
      int mx_l = max(l, np.l);
      int mn_u = min(u, np.u);
      if (mn_u < mx_l) return 1;
      else {
         return 1 - 2*((double)(mn_u - mx_l +1))/((double) (getLen() + np.getLen()));
      }
   }
}

bool NumPair::isEmpty(){
   return ((l< 0) || (u < 0) || u < l);
}

string
NumPair::str(){
   ostringstream rslt;
   rslt << "(" << l << ", " << u << ")";
   return rslt.str();
}
